# -*- coding: utf-8 -*-
from modeltranslation.translator import translator, TranslationOptions
from translator.models import Translation


class TranslationTranslationOptions(TranslationOptions):
    fields = ('description',)


translator.register(Translation, TranslationTranslationOptions)



# -*- coding: utf-8 -*-
import hashlib
import logging

from django.conf import settings
from django.core.cache.backends.base import DEFAULT_TIMEOUT
from django.db.utils import OperationalError, ProgrammingError
from django.utils.functional import lazy
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe

from translator.context_processors import DJANGO_TRANSLATOR_MODELS

CACHE_TIMEOUT = getattr(settings, "DJANGO_TRANSLATOR_CACHE_TIMEOUT", DEFAULT_TIMEOUT)


def get_translation_for_key(item, model_class=None):
    from django.core.cache import cache
    from django.core.exceptions import ObjectDoesNotExist
    from django.utils.translation import get_language

    from translator.models import Translation

    if getattr(settings, "DJANGO_TRANSLATOR_ENABLED", True):
        if not model_class:
            model_class = Translation  # We are using the Translation model as default

        lang = get_language()
        key = get_key(lang, item, model_class.cache_key_prefix)
        result = cache.get(key)

        try:
            if not result:
                try:
                    result = model_class.objects.get(key=item).description
                except ObjectDoesNotExist:
                    result = model_class(key=item)
                    result.description = item
                    result.save()
                    result = result.description

                cache.set(key, result, timeout=CACHE_TIMEOUT)
        except (OperationalError, ProgrammingError):
            logging.getLogger(__name__).info("Unable to get translation for {0}".format(item), )
            result = item
    else:
        result = item

    return mark_safe(result)


def get_key(lang, item, prefix):
    item = hashlib.sha256(item.encode('utf-8')).hexdigest()
    key = u'{0}-{1}-{2}'.format(lang, prefix, item)
    return key


def translator(key, model_key=None):
    model_class = get_model_class_by_key(model_key=model_key)
    return get_translation_for_key(item=key, model_class=model_class)


def translator_lazy(item, model_key=None):
    if len(item) == 0:
        return ''
    else:
        model_class = get_model_class_by_key(model_key=model_key)
        return lazy(get_translation_for_key, str)(item=item, model_class=model_class)


def translator_lazy_str(item):
    if len(item) == 0:
        return ''
    else:
        return lazy(get_translation_for_key, str)(item)


def get_model_class_by_key(model_key=None):
    if not model_key:
        return None

    model_class_path = DJANGO_TRANSLATOR_MODELS.get(model_key)
    try:
        return import_string(model_class_path)
    except ImportError:
        return None



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from modeltranslation.admin import TranslationAdmin

from translator.models import Translation

CATEGORY_SEPARATOR = getattr(settings, 'DJANGO_TRANSLATOR_CATEGORY_SEPARATOR', '__')


class KeyFilter(admin.SimpleListFilter):
    title = _('Categories')
    parameter_name = 'keys'

    def lookups(self, request, model_admin):
        queryset = model_admin.model.objects.filter(key__contains=CATEGORY_SEPARATOR).values_list('key', flat=True)
        unique_categories = {key.split(CATEGORY_SEPARATOR)[0] for key in queryset}
        categories = sorted([key for key in unique_categories])
        return (
            (category, category) for category in categories
        )

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(key__startswith=self.value())
        else:
            return queryset


class TranslationAdministration(TranslationAdmin):
    list_filter = (KeyFilter,)
    search_fields = ['key', 'description']
    ordering = ('key',)
    list_display = ('key', 'description')
    list_editable = ('description',)


admin.site.register(Translation, TranslationAdministration)



# coding=utf-8
from django.conf import settings
from django.core.cache import cache
from django.db import models
from django.utils.functional import classproperty

from translator.util import get_key


class TranslationBase(models.Model):
    key = models.CharField(max_length=255, primary_key=True)
    description = models.TextField()

    class Meta:
        abstract = True

    def __str__(self):
        return self.key

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        for language_code, _ in settings.LANGUAGES:
            cache.delete(get_key(language_code, self.key, self.cache_key_prefix))
        return super().save(force_insert, force_update, using, update_fields)

    @classproperty
    def cache_key_prefix(self):
        """To separate cache keys, we need to specify a unique prefix per model."""
        return self._meta.db_table


class Translation(TranslationBase):
    pass



# -*- coding: utf-8 -*-
from django import template
from django.template import Template
from translator.util import get_translation_for_key

register = template.Library()


class VariableRenderingNode(template.Node):

    def __init__(self, some_key):
        self.translation_key = template.Variable(some_key)

    def render(self, context):
        translation_for_key = get_translation_for_key(self.translation_key.resolve(context))
        return Template(translation_for_key).render(context)


def render_variable(parser, token):
    tag_name, variable_value = token.split_contents()

    # except ValueError:
    #     pass
    return VariableRenderingNode(variable_value)


register.tag('render_translation', render_variable)



# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils.module_loading import import_string

DJANGO_TRANSLATOR_MODELS = getattr(settings, 'DJANGO_TRANSLATOR_MODELS', {})


class Translator(object):

    def __init__(self, request, model_class=None):
        from translator.models import Translation

        self.request = request
        self.model_class = model_class or Translation

    def __getattr__(self, item):
        from translator.util import get_translation_for_key

        return get_translation_for_key(item=item, model_class=self.model_class)


def translator(request):
    context = {'translator': Translator(request)}
    for variable_name, model_class_path in DJANGO_TRANSLATOR_MODELS.items():
        model_class = import_string(model_class_path)
        context[variable_name] = Translator(request, model_class)
    return context



import os

from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

reqs = ['Django>=3.2', 'django-modeltranslation>=0.18.4', 'django-taggit>= 2.0.0']

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-translator',
    version='2.0.0',
    packages=['translator'],
    include_package_data=True,
    license='MIT License',
    description='Translator is an app for collecting translations for specified keys in django admin.',
    long_description=README,
    url='https://www.dreipol.ch/',
    author='dreipol GmbH',
    author_email='dev@dreipol.ch',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=reqs,
)
